import 'dart:developer';

import 'package:http/http.dart' as http;
import 'dart:convert' as convert;

late http.Response _httpResponse;

Future _respsoneConvert(val) async {
  _httpResponse = val;
  Response _kponse = Response();
  return _kponse;
}

String _headerCookies(Map<String, String> cookies) =>
    cookies.entries.map((e) => '${e.key}=${e.value}').join('; ');

class HttpRequests {
  /// An Method To Make Get HTTP Request
  /// Pass Url First Which Is Mandatory [ url like https://www.secanonm.in ] And You Can Pass Headers Using dictionary or map
  static get(String url,
      {Map<String, String>? headers, int timeout = 10}) async {
    http.Response response;
    var _url = Uri.parse(url);
    try {
      response = await http
          .get(_url, headers: headers)
          .timeout(Duration(seconds: timeout), onTimeout: () {
        log('timeout - $timeout ', name: 'HTTP_REQUESTS');
        return http.Response('Error', 500);
      });
    } catch (e) {
      log('Request failed', name: 'HTTP_REQUESTS');
      response = http.Response('Error', 500);
    }
    return _respsoneConvert(response);
  }

  /// An Method To Make Post HTTP Request
  /// Pass Url First Which Is Mandatory [ url like https://www.secanonm.in ] And You Can Pass header And Data Using dictionary or map
  static post(String url,
      {Map<String, String>? headers,
      Map<String, dynamic>? data,
      Map<String, String>? cookies,
      int timeout = 10}) async {
    var _url = Uri.parse(url);
    http.Response response;

    if (cookies != null) {
      if (data != null) {
        data['Cookie'] = _headerCookies(cookies);
      } else {
        data = {'Cookie': _headerCookies(cookies)};
      }
    }

    try {
      response = await http
          .post(_url, body: data, headers: headers)
          .timeout(Duration(seconds: timeout), onTimeout: () {
        log('timeout - $timeout ', name: 'HTTP_REQUESTS');
        return http.Response('Error', 500);
      });
    } catch (e) {
      log('Request failed', name: 'HTTP_REQUESTS');
      response = http.Response('Error', 500);
    }

    return await _respsoneConvert(response);
  }
}

/// Simply Class To Handle Response
class Response {
  /// Simply Return Response code
  bool get hasError =>
      (400 <= _httpResponse.statusCode) && (_httpResponse.statusCode < 600);

  /// It return success status of request ,if request is complete than it return true else return false
  bool get success => !hasError;

  /// It return HTTP status Code of response like 200 , 301, 502
  get status => _httpResponse.statusCode;

  /// It return Content Type of response like json or text
  String get contentType => _httpResponse.headers['content-type'] ?? "undefine";

  /// It return content of response
  String get content => convert.utf8.decode(bytes, allowMalformed: true);

  /// It return json response in format of Map { like in python dict }
  Map<String, dynamic> get json {
    Map<String, dynamic> _helo;
    try {
      _helo = convert.jsonDecode(_httpResponse.body) as Map<String, dynamic>;
    } catch (e) {
      log('Failed to convert JSON', name: 'HTTP_REQUESTS');
      _helo = {"status": "failed"};
    }
    return _helo;
  }

  /// It return response header in format of Map { like in python dict }
  get headers => _httpResponse.headers;

  /// It return content of response simply
  get response => _httpResponse.body;

  /// It return content length of response simply
  get contentLength => _httpResponse.contentLength;

  /// It return Redirection Status of response simply
  get isRedirect => _httpResponse.isRedirect;

  /// It return bytes comprising the body of response
  List<int> get bytes => _httpResponse.bodyBytes;
}
